{% extends 'basic/layout.html' %}
{% block extrahead %}
<link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" integrity="sha384-wvfXpqpZZVQGK6TAh5PVlGOfQNHSoD2xbE+QkPxCAFlNEevoEH3Sl0sibVcOQVnN" crossorigin="anonymous">
<link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.5.0/css/bootstrap.min.css" integrity="sha384-9aIt2nRpC12Uk9gS9baDl411NQApFmC26EwAOH8WgZl5MYYxFfc+NcPb1dKGj7Sk" crossorigin="anonymous">
<script src="https://cdn.jsdelivr.net/npm/popper.js@1.16.0/dist/umd/popper.min.js" integrity="sha384-Q6E9RHvbIyZFJoft+2mJbHaEWldlvI9IOYy5n3zV9zzTtmI3UksdQRVvoxMfooAo" crossorigin="anonymous"></script>
<script src="https://stackpath.bootstrapcdn.com/bootstrap/4.5.0/js/bootstrap.min.js" integrity="sha384-OgVRvuATP1z7JjHLkuOU7Xw704+h835Lr+6QL9UvYjZE3Ipu6Tp75j7Bh/kR0JKI" crossorigin="anonymous"></script>
{% endblock %}

{# https://www.sphinx-doc.org/en/master/templating.html #}

{% block relbar1 %}{% if pagename != 'index' %}{{ relbar() }}{% endif %}{% endblock %}

{% block relbar2 %}
{% endblock %}

{% block header %}
    {% if pagename == 'index' %}<div class=indexwrapper>{% endif %}
{% endblock %}

{% block sidebar1 %}
{% endblock %}

{% block sidebar2 %}
 {%- if render_sidebar %}
  <div class="sphinxsidebar" role="navigation" aria-label="main navigation">
    <div class="sphinxsidebarwrapper">
      {%- block sidebarlogo %}
      {%- if logo %}
        <p class="logo"><a href="https://cutter.re/">
          <img class="logo" src="{{ pathto('_static/' + logo, 1) }}" alt="Logo"/>
        </a></p>
      {%- endif %}
      {%- endblock %}
      <a href="{{ pathto(master_doc) }}">{{ project }} documentation</a>
      {%- if sidebars != None %}
        {#- new style sidebar: explicitly include/exclude templates #}
        {%- for sidebartemplate in sidebars %}
        {%- include sidebartemplate %}
        {%- endfor %}
      {%- else %}
        {#- old style sidebars: using blocks -- should be deprecated #}
      {%- endif %}
    </div>
  </div>
 {%- endif %}
{% endblock %}

{% block footer %}
    {{ super() }}
    {% if pagename == 'index' %}</div>{% endif %}
{% endblock %}
